package com.bocloud.ams.booter.controller.collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.collection.CollectionTask;
import com.bocloud.ams.entity.model.OperateParam;
import com.bocloud.ams.service.collection.CollectionTaskService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/collection/tasks")
@Tag(name = "采集任务管理")
public class CollectionTaskController implements ApplicationListener<ApplicationReadyEvent> {

    @Autowired
    private CollectionTaskService collectionTaskService;

    /**
     * 启动作业
     *
     * @param id
     * @param context
     * @return
     */
    @PatchMapping(value = "/{id}/start")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集任务执行")
    public GeneralResult<Long> start(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTaskService.start(id, context);
    }

    /**
     * 采集任务列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "采集任务列表")
    @GetMapping
    public GeneralResult<GridBean<CollectionTask>> list(Pager pager) {
        return collectionTaskService.list(pager);
    }

    /**
     * 采集任务创建
     *
     * @param task
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC"}, summary = "采集任务创建")
    public GeneralResult<CollectionTask> create(@RequestBody CollectionTask task,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTaskService.save(task, context);
    }

    /**
     * 采集任务修改
     * 
     * @param bean
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "采集任务修改")
    public GeneralResult<CollectionTask> modify(@RequestBody CollectionTask bean,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTaskService.modify(bean, context);
    }

    /**
     * 采集任务类型变更->由定时任务变更为常规任务
     * 
     * @param context
     * @return
     */
    @PatchMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "采集任务定时任务转为常规任务")
    public GeneralResult<CollectionTask> modifyType(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTaskService.modifyType(id, context);
    }

    /**
     * 采集任务删除
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集任务删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionTaskService.remove(id, context.getTarget());
    }

    /**
     * 采集任务详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集任务详情")
    public GeneralResult<CollectionTask> detail(@PathVariable(value = Common.ID) Long id) {
        return collectionTaskService.detail(id);
    }

    /**
     * 采集定时任务操作-启动/暂停
     * 
     * @param operateParam
     * @param requestContext
     * @return
     */
    @PatchMapping(value = "/{id}/schedule")
    @Operation(summary = "采集任务定时任务操作-启动/暂停")
    public GeneralResult<Void> operate(@RequestBody OperateParam operateParam, @PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return collectionTaskService.operate(operateParam, id, requestContext.getTarget());
    }

    @GetMapping(value = "/{id}/schedule")
    @Operation(tags = {"CMC"}, summary = "采集任务为定时采集添加任务(主服务有效)")
    public GeneralResult<String> addScheduleIfLeader(@PathVariable(Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return collectionTaskService.addScheduleIfLeader(id);
    }

    @DeleteMapping(value = "/{id}/schedule")
    @Operation(tags = {"CMC"}, summary = "采集任务为定时采集删除任务(主服务有效)")
    public GeneralResult<String> removeScheduleIfLeader(@PathVariable(Common.ID) Long id) {
        return collectionTaskService.removeScheduleIfLeader(id);
    }

    @Override
    public void onApplicationEvent(ApplicationReadyEvent event) {
        collectionTaskService.onApplicationReadyEvent();
    }
}
