package com.bocloud.ams.booter.controller.module;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.model.PropertyModel;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.service.association.AutoRelationRuleService;
import com.bocloud.ams.service.module.ModuleService;
import com.bocloud.ams.service.module.RelationCategoryService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/module")
@Tag(name = "模型管理")
public class ModuleController {

    @Autowired
    private ModuleService moduleService;
    @Autowired
    private RelationCategoryService relationCategoryService;
    @Autowired
    private AutoRelationRuleService autoRelationRuleService;

    /**
     * 查询属性池记录
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/collections")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型列表")
    public GeneralResult<GridBean<CiModule>> getModelList(Pager pager,
                                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.getModelList(pager);
    }

    /**
     * 查询模型集合
     *
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC"}, summary = "模型列表（全部）")
    public GeneralResult<List<CiModule>> list() {
        return moduleService.list();
    }

    /**
     * 模型查询明细
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "模型详情")
    public GeneralResult<CiModule> detail(@PathVariable(value = Common.ID) Long id) {
        return moduleService.detail(id);
    }

    /**
     * 新增模型
     *
     * @param module
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC"}, summary = "模型新增")
    public GeneralResult<Void> save(@RequestBody CiModule module, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.save(module, context.getTarget());
    }

    /**
     * 更新模型
     *
     * @param module
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "模型修改")
    public GeneralResult<Void> modify(@RequestBody CiModule module, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.modify(module, context.getTarget());
    }

    /**
     * 删除模型
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "模型删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.remove(id, context.getTarget());
    }

    /**
     * 根据模型Id查询关联分组
     * 
     * @param id
     *            模型id
     * @Description: 根据模型Id查询关联分组
     * @Author majiali
     * @CreateTime 2022/6/15 16:07
     */
    @GetMapping(value = "/{id}/relation/category")
    @Operation(tags = {"CMC"}, summary = "模型关联分组查询")
    public GeneralResult<List<RelationCategory>> listRelationCategory(@PathVariable(value = Common.ID) Long id) {
        return relationCategoryService.listByModuleId(id);
    }

    /**
     * 根据模型id和关联分组id查询关联规则
     *
     * @param id
     * @param relationCategoryId
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "模型关联分组关联规则查询")
    @GetMapping(value = "/{id}/relation/rule")
    public GeneralResult<List<AutoRelationRule>> list(@PathVariable(value = Common.ID) Long id,
        @RequestParam(value = "relationCategoryId") Long relationCategoryId) {
        return autoRelationRuleService.list(id, relationCategoryId);
    }

    /**
     * 查看模型属性
     *
     * @param id
     * @return 查看模型属性
     */
    @GetMapping(value = "/{id}/properties")
    @Operation(summary = "模型属性查询")
    public GeneralResult<PropertyModel> property(@PathVariable(value = Common.ID) Long id) {
        return moduleService.property(id);
    }

    /**
     * 查询模型属性集合
     *
     * @param moduleId
     * @param pager
     * @return
     */
    @GetMapping(value = "/{moduleId}/list")
    @Operation(summary = "模型属性列表")
    public GeneralResult<GridBean<Property>> propertyList(@PathVariable(value = "moduleId") Long moduleId,
        Pager pager) {
        return moduleService.propertyList(moduleId, pager);
    }

    /**
     * 编辑模型属性
     * 
     * @param model
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}/properties")
    @Operation(summary = "模型属性修改")
    public GeneralResult<Void> modifyProperty(@RequestBody PropertyModel model,
        @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.modifyProperty(model, id, context);
    }

    /**
     * 首页统计模型总数-服务器、虚拟机、网络设备、安全设别
     *
     * @param model
     * @param id
     * @param context
     * @return
     */
    @GetMapping(value = "/statistics")
    @Operation(summary = "首页统计模型总数")
    public GeneralResult statistics(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleService.statistics(context);
    }

}
