package com.bocloud.ams.repository.instance;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author yangzhenrong
 * @Date 2022/5/11 11:31
 * @Version 1.0
 **/
@Repository
public class InstanceValueRepository extends BasicGenericDao<InstanceValue, Long> {

    public InstanceValueRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public void createTable(String instanceValueTable, InstanceValue cmdbInstanceValue) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + instanceValueTable + " (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(cmdbInstanceValue);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL AUTO_INCREMENT,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        this.execute(sqlBuilder.toString(), null);
    }

    public void dropTable(String instanceValueTable) throws Exception {
        String sql = "DROP TABLE " + instanceValueTable;
        this.execute(sql, null);
    }

    public int countTable(String instanceValueTable) throws Exception {
        String sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :instanceValueTable;";
        Map<String, Object> params = MapTools.simpleMap("instanceValueTable", instanceValueTable);
        return this.countQuery(sql, params).intValue();
    }

    public List<InstanceValue> list(Pager pager, String instanceValueTable) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InstanceValue.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params, String instanceValueTable) throws Exception {
        String sql = "select count(1) from " + instanceValueTable + " a where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<InstanceValue> listByInstanceId(Long instanceId, String instanceValueTable) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where a.is_deleted=0 and a.instance_id=:instanceId";
        return this.list(InstanceValue.class, sql, MapTools.simpleMap("instanceId", instanceId));
    }

    public boolean remove(Long id, String instanceValueTable, Long userId) throws Exception {
        String sql = "update " + instanceValueTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByInstanceId(Long instanceId, String instanceValueTable, Long userId) throws Exception {
        String sql = "update " + instanceValueTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByPropertyId(Long propertyId, String instanceValueTable, Long userId) throws Exception {
        String sql = "update " + instanceValueTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where property_id = :propertyId ";
        Map<String, Object> params = MapTools.simpleMap("propertyId", propertyId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public InstanceValue query(String instanceValueTable, String propertyCode, String propertyValue) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where property_code = :propertyCode and property_value = :propertyValue and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("propertyCode", propertyCode);
        params.put("propertyValue", propertyValue);
        List<InstanceValue> list = this.list(InstanceValue.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public InstanceValue query(String instanceValueTable, String propertyCode, Long instanceId) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where property_code = :propertyCode and instance_id = :instanceId and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("propertyCode", propertyCode);
        params.put("instanceId", instanceId);
        List<InstanceValue> list = this.list(InstanceValue.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InstanceValue> queryByInstanceIdsAndPropertyIds(String instanceValueTable, List<Long> instanceIds, List<Long> propertyIds) throws Exception {
        final String sql = "select a.* from " + instanceValueTable + " a where a.instance_id in ( :instanceIds ) and a.property_id in ( :propertyIds ) and a.is_deleted = 0 ";
        final Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        params.put("propertyIds", propertyIds);
        return this.list(InstanceValue.class, sql, params);
    }

    public List<InstanceValue> listByPropertyCode(String instanceValueTable, String propertyCode) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where a.property_code = :propertyCode and a.is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("propertyCode", propertyCode);
        return this.list(InstanceValue.class, sql, params);
    }

    public List<InstanceValue> queryByInstanceIds(String instanceTableName, List<Long> instanceIdList) {
        String sql = "select a.* from " + instanceTableName + " a where a.instance_id in (:instanceIdList) and a.is_deleted = 0 ";
        return this.list(InstanceValue.class, sql, MapTools.simpleMap("instanceIdList", instanceIdList));
    }

    public List<InstanceValue> listAll(String instanceValueTable) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where a.is_deleted=0 ";
        return this.list(InstanceValue.class, sql, null);
    }

    public boolean batchRemoveByInstanceIds(List<Long> instanceIds, String instanceValueTable, Long userId) throws Exception {
        String sql = "update " + instanceValueTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<InstanceValue> queryList(String instanceValueTable, String propertyCode, List<Long> instanceIds) throws Exception {
        String sql = "select a.* from " + instanceValueTable + " a where property_code = :propertyCode and instance_id in (:instanceIds) and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("propertyCode", propertyCode);
        params.put("instanceIds", instanceIds);
        return this.list(InstanceValue.class, sql, params);
    }

}
