package com.bocloud.ams.service.module.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.PropertyItem;
import com.bocloud.ams.entity.module.PropertyPool;
import com.bocloud.ams.entity.module.PropertyPoolItem;
import com.bocloud.ams.repository.module.PropertyItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.service.module.PropertyPoolService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Description: 属性池实现类
 * @Title: PropertyPoolServiceImpl
 * @Author: majiali
 * @CreateTime: 2022/5/5 14:14
 */
@Slf4j
@Service("propertyPoolService")
public class PropertyPoolServiceImpl implements PropertyPoolService {

    @Autowired
    private PropertyPoolRepository propertyPoolRepository;
    @Autowired
    private PropertyPoolItemRepository propertyPoolItemRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private PropertyItemRepository propertyItemRepository;

    @Override
    public GeneralResult create(PropertyPool pool, Long userId) {
        try {
            if (judgeCodeExist(pool.getCode())) {
                return new GeneralResult(false, "属性编码已经存在");
            }
            if (PropertyConstant.RESERVED_MODULE_CODES.contains(pool.getCode().toLowerCase())) {
                throw new Exception("属性编码[" + pool.getCode() + "]为系统内置编码，请修改后重试");
            }
            validate(pool);
            pool.setDeleted(false);
            pool.setCreatorId(userId);
            pool.setMenderId(userId);
            pool.setGmtModify(new Date());
            pool.setGmtCreate(new Date());
            propertyPoolRepository.save(pool);
            // 非引用类型的属性item才有值
            if (null == pool.getLinkModuleId()) {
                List<PropertyPoolItem> items = pool.getItemList();
                if (CollectionUtils.isNotEmpty(items)) {
                    for (PropertyPoolItem item : items) {
                        item.setDeleted(false);
                        item.setCreatorId(userId);
                        item.setMenderId(userId);
                        item.setGmtModify(new Date());
                        item.setGmtCreate(new Date());
                        item.setPropertyPoolId(pool.getId());
                        List<PropertyPoolItem> itemValueList = item.getItemList();
                        if (CollectionUtils.isNotEmpty(itemValueList)) {
                            item.setItemValue(JSONObject.toJSONString(itemValueList));
                        }
                    }
                    propertyPoolItemRepository.batchSave(items, 100);
                }
            }
            return new GeneralResult<>(true, "属性新增成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool create fail:", e);
            return new GeneralResult<>(false, "属性新增失败," + e.getMessage());
        }
    }

    private boolean judgeCodeExist(String code) throws Exception {
        PropertyPool temp = propertyPoolRepository.queryByCode(code);
        return null != temp;
    }

    /**
     * 数据校验
     *
     * @param pool
     * @throws Exception
     */
    private void validate(PropertyPool pool) throws Exception {
        String code = pool.getCode().toLowerCase().trim();
        Pattern compile = Pattern.compile(
            "^(create|alter|select|update|union|and|or|delete|insert|trancate|char|substr|commit|ascii|declare|exec|count|show|into|drop|execute)$");
        if (compile.matcher(code).find()) {
            throw new Exception("属性编码不能是关键字：" + code);
        }
        if (!code.matches("^[a-zA-Z][0-9a-zA-Z_]{0,}$")) {
            throw new Exception("唯一标识必须以字母开头且由字母数字或者下划线组成");
        }
        List<PropertyPoolItem> items = pool.getItemList();
        if (CollectionUtils.isEmpty(items)) {
            return;
        }
        if (pool.getType().equals(PropertyConstant.PropertyType.MULTILINETABLE)
            || pool.getType().equals(PropertyConstant.PropertyType.SINGLELINETABLE)) {
            Map<String, String> itemNameMap = new HashMap<>();
            Map<String, String> itemValueMap = new HashMap<>();
            for (PropertyPoolItem tableItem : items) {
                String itemName = tableItem.getName();
                if (StringUtils.isEmpty(itemName)) {
                    throw new Exception("结构项名称不能为空！");
                }
                if (itemNameMap.get(itemName) != null) {
                    throw new Exception("结构项名称重复：" + itemName);
                }
                itemNameMap.put(itemName, itemName);
                String itemValue = tableItem.getCode();
                if (StringUtils.isEmpty(itemValue)) {
                    throw new Exception("结构项ID不能为空");
                }
                if (itemValueMap.get(itemValue) != null) {
                    throw new Exception("结构项ID重复：" + itemValue);
                }
                itemValueMap.put(itemValue, itemValue);

                if (tableItem.getType().equals(PropertyConstant.PropertyType.SINGLESELECT)
                    || tableItem.getType().equals(PropertyConstant.PropertyType.MULTISELECT)
                    || tableItem.getType().equals(PropertyConstant.PropertyType.RADIO)
                    || tableItem.getType().equals(PropertyConstant.PropertyType.CHECKBOX)) {
                    validateItemValues(tableItem);
                }
            }
        } else {

            Map<String, String> itemNameMap = new HashMap<>();
            Map<String, String> itemValueMap = new HashMap<>();
            for (PropertyPoolItem item : items) {
                String itemName = item.getName();
                if (StringUtils.isEmpty(itemName)) {
                    throw new Exception("枚举值描述不能为空！");
                }
                if (itemNameMap.get(itemName) != null) {
                    // throw new Exception("属性选项名称重复：" + itemName);
                    throw new Exception("枚举值描述必须唯一");
                }
                itemNameMap.put(itemName, itemName);
                String itemValue = item.getValue();
                if (StringUtils.isEmpty(itemValue)) {
                    throw new Exception("枚举值不能为空");
                }
                if (itemValueMap.get(itemValue) != null) {
                    // throw new Exception("属性选项值重复：" + itemValue);
                    throw new Exception("枚举值必须唯一");
                }
                itemValueMap.put(itemValue, itemValue);
            }
        }
    }

    private void validateItemValues(PropertyPoolItem tableItem) throws Exception {
        List<PropertyPoolItem> itemValues = tableItem.getItemList();
        Map<String, String> itemValueMap = new HashMap<>();
        Map<String, String> itemNameMap = new HashMap<>();
        for (PropertyPoolItem item : itemValues) {
            String itemName = item.getName();
            if (StringUtils.isEmpty(itemName)) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值描述不能为空");
            }
            if (itemNameMap.get(itemName) != null) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值描述必须唯一");
            }
            itemNameMap.put(itemName, itemName);
            String itemValue = item.getValue();
            if (StringUtils.isEmpty(itemValue)) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值不能为空");
            }
            if (itemValueMap.get(itemValue) != null) {
                // throw new Exception("结构项【" + tableItem.getName() + "】的属性选项值重复：" + itemValue);
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值必须唯一");
            }
            itemValueMap.put(itemValue, itemValue);
        }
    }

    @Override
    public GeneralResult modify(PropertyPool pool, Long userId) {
        String path = PropertyPoolServiceImpl.class.getSimpleName() + "_" + pool.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PropertyPool temp = propertyPoolRepository.query(pool.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "属性不存在");
            }
            validate(pool);
            LocaleBeanUtils.copyProperties(temp, pool);
            temp.setGmtModify(new Date());
            temp.setMenderId(userId);
            propertyPoolRepository.update(temp);
            // 非引用类型的属性item才有值
            if (null == pool.getLinkModuleId()) {
                // 更新配置项
                updatePoolItem(pool, userId);
            }
            return new GeneralResult<>(true, "属性修改成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool modify fail:", e);
            return new GeneralResult<>(false, "属性修改失败," + e.getMessage());
        }
    }

    private void updatePoolItem(PropertyPool pool, Long userId) throws Exception {
        List<PropertyPoolItem> itemList = pool.getItemList();
        List<PropertyPoolItem> oldItemList = propertyPoolItemRepository.listByPropertyPoolId(pool.getId());
        if (CollectionUtils.isEmpty(oldItemList)) {
            if (CollectionUtils.isNotEmpty(itemList)) {
                itemList.forEach(item -> {
                    item.setDeleted(false);
                    item.setCreatorId(userId);
                    item.setMenderId(userId);
                    item.setGmtModify(new Date());
                    item.setGmtCreate(new Date());
                    item.setPropertyPoolId(pool.getId());
                    List<PropertyPoolItem> itemValueList = item.getItemList();
                    if (CollectionUtils.isNotEmpty(itemValueList)) {
                        item.setItemValue(JSONObject.toJSONString(itemValueList));
                    }
                });
                propertyPoolItemRepository.batchSave(itemList, 100);
            }
        } else {
            if (CollectionUtils.isNotEmpty(itemList)) {
                List<PropertyPoolItem> addList = new ArrayList<>();
                List<PropertyPoolItem> updateList = new ArrayList<>();
                Map<Long, List<PropertyPoolItem>> itemMap =
                    oldItemList.stream().collect(Collectors.groupingBy(PropertyPoolItem::getId));
                itemList.forEach(item -> {
                    if (null == item.getId() || 0L == item.getId()) {
                        item.setDeleted(false);
                        item.setCreatorId(userId);
                        item.setMenderId(userId);
                        item.setGmtModify(new Date());
                        item.setGmtCreate(new Date());
                        item.setPropertyPoolId(pool.getId());
                        List<PropertyPoolItem> itemValueList = item.getItemList();
                        if (CollectionUtils.isNotEmpty(itemValueList)) {
                            item.setItemValue(JSONObject.toJSONString(itemValueList));
                        }
                        addList.add(item);
                    } else {
                        itemMap.remove(item.getId());
                        item.setMenderId(userId);
                        item.setGmtModify(new Date());
                        item.setPropertyPoolId(pool.getId());
                        List<PropertyPoolItem> itemValueList = item.getItemList();
                        if (CollectionUtils.isNotEmpty(itemValueList)) {
                            item.setItemValue(JSONObject.toJSONString(itemValueList));
                        }
                        updateList.add(item);
                    }
                });
                if (CollectionUtils.isNotEmpty(addList)) {
                    propertyPoolItemRepository.batchSave(addList, 100);
                }
                if (CollectionUtils.isNotEmpty(updateList)) {
                    propertyPoolItemRepository.batchUpdate(updateList, 100);
                }
                if (!itemMap.isEmpty()) {
                    propertyPoolItemRepository.deleteByIds(new ArrayList<>(itemMap.keySet()), userId);
                }
            } else {
                propertyPoolItemRepository.deleteByPropertyPoolId(pool.getId(), userId);
            }
        }
    }

    @Override
    public GeneralResult remove(Long id, Long userId) {
        String path = PropertyPoolServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PropertyPool temp = propertyPoolRepository.query(id);
            if (null == temp) {
                return new GeneralResult<>(false, "属性不存在");
            }
            final List<Property> propertyList = propertyRepository.queryListByPoolIds(id);
            if (CollectionUtils.isNotEmpty(propertyList)) {
                return new GeneralResult<>(false, "该属性已关联模型，请先解除关联");
            }
            propertyPoolItemRepository.deleteByPropertyPoolId(id, userId);
            propertyPoolRepository.delete(id, userId);
            return new GeneralResult<>(true, "属性删除成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool remove fail:", e);
            return new GeneralResult<>(false, "属性删除失败");
        }
    }

    @Override
    public GeneralResult detail(Long id) {
        try {
            PropertyPool pool = propertyPoolRepository.query(id);
            if (null == pool) {
                return new GeneralResult<>(false, "属性查询失败，数据不存在");
            }
            dealPoolItem(pool);
            return new GeneralResult<>(true, pool, "属性查询成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool detail query fail:", e);
            return new GeneralResult<>(false, "属性查询失败");
        }
    }

    @Override
    public GeneralResult list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = propertyPoolRepository.count(pager.getParams());
            List<PropertyPool> list = this.propertyPoolRepository.list(pager);
            if (CollectionUtils.isNotEmpty(list)) {
                for (PropertyPool pool : list) {
                    dealPoolItem(pool);
                }
            }
            GridBean<PropertyPool> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void dealPoolItem(PropertyPool pool) throws Exception {
        if (PropertyConstant.TABLE_TYPE_ONE.equals(pool.getTableType())) {
            List<Long> propertyIds = Arrays.asList(pool.getPropertyIds().split(",")).stream()
                .map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            // 查询不是引入属性池中的属性信息
            List<PropertyPoolItem> poolItems = new ArrayList<>();
            List<Property> properties = propertyRepository.queryListNotLink(propertyIds);
            if (CollectionUtils.isNotEmpty(properties)) {
                // 查询属性项
                List<PropertyItem> items =
                    propertyItemRepository.queryByModuleIdAndPropertyIds(pool.getLinkModuleId(), propertyIds);
                Map<Long, List<PropertyItem>> itemsMap =
                    items.stream().collect(Collectors.groupingBy(PropertyItem::getPropertyId));
                PropertyPoolItem item;
                for (Property property : properties) {
                    // 查询属性项
                    item = new PropertyPoolItem();
                    item.setName(property.getName());
                    item.setType(property.getType());
                    item.setCode(property.getCode());
                    List<PropertyItem> propertyItems = itemsMap.get(property.getId());
                    if (CollectionUtils.isNotEmpty(propertyItems)) {
                        item.setItemList(
                            JSONObject.parseArray(JSONObject.toJSONString(propertyItems), PropertyPoolItem.class));
                    }
                    item.setPropertyId(property.getId());
                    item.setModuleId(property.getModuleId());
                    item.setPropertyPoolId(pool.getId());
                    poolItems.add(item);
                }
            }
            // 查询引入属性池中的属性信息
            List<PropertyPool> pools = propertyPoolRepository.listByPropertyIds(propertyIds);
            if (CollectionUtils.isNotEmpty(pools)) {
                List<Long> poolIds = pools.stream().map(PropertyPool::getId).collect(Collectors.toList());
                // 查询属性项
                List<PropertyPoolItem> items = propertyPoolItemRepository.listByPropertyPoolIds(poolIds);
                Map<Long, List<PropertyPoolItem>> itemsMap =
                    items.stream().collect(Collectors.groupingBy(PropertyPoolItem::getPropertyPoolId));
                for (PropertyPool property : pools) {
                    // 查询属性项
                    PropertyPoolItem item = new PropertyPoolItem();
                    item.setName(property.getName());
                    item.setType(property.getType());
                    item.setCode(property.getCode());
                    item.setItemList(itemsMap.get(property.getId()));
                    item.setPropertyId(property.getId());
                    item.setPropertyPoolId(pool.getId());
                    poolItems.add(item);
                }
            }
            pool.setItemList(poolItems);
        } else {
            List<PropertyPoolItem> items = propertyPoolItemRepository.listByPropertyPoolId(pool.getId());
            if (CollectionUtils.isNotEmpty(items)) {
                for (PropertyPoolItem item : items) {
                    if (StringUtils.isNotEmpty(item.getItemValue())) {
                        item.setItemList(JSONObject.parseArray(item.getItemValue(), PropertyPoolItem.class));
                    }
                }
            }
            pool.setItemList(items);
        }
    }

}
