package com.bocloud.ams.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.entity.association.AutoRelationRuleItemValue;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceRelationHistoryRecord;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.InstanceRelationHistoryAlterModel;
import com.bocloud.ams.entity.module.ModuleRelation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description: 判断两个实例是否有关系工具类
 * @Title: JudgeInstanceRelationUtils
 * @Author: majiali
 * @CreateTime: 2022/6/9 15:19
 */
@Slf4j
public class AutoModuleInstanceRelationUtils {

    /**
     * @param sourceInstance 当前实例信息（源）
     * @param targetInstance 目标实例
     * @param relationRule   关联规则
     * @Description:判断两个实例是否有关系 正向判断
     * @Author majiali
     * @CreateTime 2022/5/20 16:35
     */
    public static boolean isInstanceRelation(Instance sourceInstance, Instance targetInstance, AutoRelationRule relationRule) {
        List<InstanceValue> instanceValues = sourceInstance.getInstanceValues();
        if (CollectionUtils.isEmpty(instanceValues)) {
            return false;
        }
        Map<String, InstanceValue> sourceValueMap = instanceValues.stream().collect(Collectors.toMap(InstanceValue::getPropertyCode, Function.identity(), (oldData, newData) -> newData));
        List<InstanceValue> targetValues = targetInstance.getInstanceValues();
        if (CollectionUtils.isEmpty(targetValues)) {
            return false;
        }
        Map<String, InstanceValue> targetValueMap = targetValues.stream().collect(Collectors.toMap(InstanceValue::getPropertyCode, Function.identity(), (oldData, newData) -> newData));
        //规则项
        List<AutoRelationRuleItem> items = relationRule.getItems();
        boolean flag = true;
        for (AutoRelationRuleItem item : items) {
            if (!flag) {
                break;
            }
            String sourceFieldCode = item.getSourceFieldCode();
            String sourceFieldType = item.getSourceFieldType();
            String targetFieldCode = item.getTargetFieldCode();
            String targetFieldType = item.getTargetFieldType();
            InstanceValue sourceValue = sourceValueMap.get(sourceFieldCode);
            InstanceValue targetValue = targetValueMap.get(targetFieldCode);
            if (null == sourceValue || StringUtils.isEmpty(sourceValue.getPropertyValue()) || null == targetValue || StringUtils.isEmpty(targetValue.getPropertyValue())) {
                flag = false;
                break;
            }
            //规则项value
            List<AutoRelationRuleItemValue> values = item.getValues();
            if (sourceFieldType.equals(PropertyConstant.PropertyType.SINGLELINETABLE) || sourceFieldType.equals(PropertyConstant.PropertyType.MULTILINETABLE)
                    || targetFieldType.equals(PropertyConstant.PropertyType.SINGLELINETABLE) || targetFieldType.equals(PropertyConstant.PropertyType.MULTILINETABLE)) {
                if (CollectionUtils.isEmpty(values)) {
                    flag = false;
                    break;
                }
            }
            //屬性行值
            String sourcePropertyValue = sourceValue.getPropertyValue();
            String targetPropertyValue = targetValue.getPropertyValue();
            //单行表格值为  JSONArray 类型
            //多行表格值为 JSONArray 类型
            switch (sourceFieldType) {
                case PropertyConstant.PropertyType.SINGLELINETABLE:
                case PropertyConstant.PropertyType.MULTILINETABLE:
                    switch (targetFieldType) {
                        case PropertyConstant.PropertyType.SINGLELINETABLE:
                        case PropertyConstant.PropertyType.MULTILINETABLE:
                            flag = multiTableCompareMultiTabel(sourcePropertyValue, targetPropertyValue, values);
                            break;
                        case PropertyConstant.PropertyType.MULTISELECT:
                        case PropertyConstant.PropertyType.CHECKBOX:
                            try {
                                JSONArray sourceArray = JSONArray.parseArray(sourcePropertyValue);
                                //目标为单个字段，规则字段只有一个
                                AutoRelationRuleItemValue value = values.get(0);
                                flag = false;
                                for (Object a : sourceArray) {
                                    JSONObject obj = (JSONObject) JSON.toJSON(a);
                                    String sourcePropertyFieldValue = obj.getString(value.getSourceFieldCode());
                                    flag = compareCheckbox(sourcePropertyFieldValue, targetPropertyValue);
                                    if (flag) {
                                        break;
                                    }
                                }

                            } catch (Exception e) {
                                log.error("目标数据格式异常", targetPropertyValue);
                                flag = false;
                            }
                            break;
                        default:
                            try {
                                JSONArray sourceArray = JSONArray.parseArray(sourcePropertyValue);
                                //目标为单个字段，规则字段只有一个
                                AutoRelationRuleItemValue value = values.get(0);
                                flag = false;
                                for (Object a : sourceArray) {
                                    JSONObject obj = (JSONObject) JSON.toJSON(a);
                                    String sourcePropertyFieldValue = obj.getString(value.getSourceFieldCode());
                                    if (StringUtils.isNotEmpty(sourcePropertyFieldValue) && sourcePropertyFieldValue.equals(targetPropertyValue)) {
                                        flag = true;
                                        break;
                                    }
                                }
                            } catch (Exception e) {
                                log.error("目标数据格式异常", targetPropertyValue);
                                flag = false;
                            }
                            break;
                    }
                    break;
                //源多选
                case PropertyConstant.PropertyType.MULTISELECT:
                case PropertyConstant.PropertyType.CHECKBOX:
                    switch (targetFieldType) {
                        case PropertyConstant.PropertyType.SINGLELINETABLE:
                        case PropertyConstant.PropertyType.MULTILINETABLE:
                            try {
                                JSONArray targetArray = JSONArray.parseArray(targetPropertyValue);
                                //源为单个字段，规则字段只有一个
                                AutoRelationRuleItemValue value = values.get(0);
                                flag = false;
                                for (Object a : targetArray) {
                                    JSONObject obj = (JSONObject) JSON.toJSON(a);
                                    String targetPropertyFieldValue = obj.getString(value.getTargetFieldCode());
                                    flag = compareCheckbox(sourcePropertyValue, targetPropertyFieldValue);
                                    if (flag) {
                                        break;
                                    }
                                }
                            } catch (Exception e) {
                                log.error("目标数据格式异常", targetPropertyValue);
                                flag = false;
                            }
                            break;
                        default:
                            //直接比较是否有一个选择项相同
                            flag = compareCheckbox(sourcePropertyValue, targetPropertyValue);
                            break;
                    }
                    break;
                default:
                    switch (targetFieldType) {
                        case PropertyConstant.PropertyType.SINGLELINETABLE:
                        case PropertyConstant.PropertyType.MULTILINETABLE:
                            try {
                                JSONArray targetArray = JSONArray.parseArray(targetPropertyValue);
                                //源为单个字段，规则字段只有一个
                                AutoRelationRuleItemValue value = values.get(0);
                                flag = false;
                                for (Object a : targetArray) {
                                    JSONObject obj = (JSONObject) JSON.toJSON(a);
                                    String targetPropertyFieldValue = obj.getString(value.getTargetFieldCode());
                                    if (StringUtils.isEmpty(targetPropertyFieldValue)) {
                                        continue;
                                    }
                                    if (PropertyConstant.PropertyType.JSON.equals(sourceFieldType)) {
                                        //判断json值是否一样
                                        flag = compareJson(sourcePropertyValue, targetPropertyValue);
                                        if (flag) {
                                            break;
                                        }
                                    } else {
                                        if (sourcePropertyValue.equals(targetPropertyFieldValue)) {
                                            flag = true;
                                            break;
                                        }
                                    }

                                }

                            } catch (Exception e) {
                                log.error("目标数据格式异常", targetPropertyValue);
                                flag = false;
                            }
                            break;
                        default:
                            if (PropertyConstant.PropertyType.JSON.equals(sourceFieldType)) {
                                //判断json值是否一样
                                flag = compareJson(sourcePropertyValue, targetPropertyValue);
                            } else {
                                if (!sourcePropertyValue.equals(targetPropertyValue)) {
                                    flag = false;
                                }
                            }
                            break;
                    }
                    break;
            }
        }
        return flag;
    }

    /**
     * 多行表格与多行表格比较
     */
    private static boolean multiTableCompareMultiTabel(String sourcePropertyValue, String targetPropertyValue, List<AutoRelationRuleItemValue> values) {
        boolean flag = true;
        try {
            JSONArray targetArray = JSONArray.parseArray(targetPropertyValue);
            JSONArray sourceArray = JSONArray.parseArray(sourcePropertyValue);
            for (AutoRelationRuleItemValue value : values) {
                //一个规则的校验结果
                boolean singleResult = false;
                for (Object a : sourceArray) {
                    JSONObject sourceObj = (JSONObject) JSON.toJSON(a);
                    String sourceValue = sourceObj.getString(value.getSourceFieldCode());
                    for (Object b : targetArray) {
                        JSONObject targetObj = (JSONObject) JSON.toJSON(b);
                        String targetValue = targetObj.getString(value.getTargetFieldCode());
                        if (PropertyConstant.PropertyType.CHECKBOX.equals(value.getSourceFieldType()) || PropertyConstant.PropertyType.MULTISELECT.equals(value.getSourceFieldType())) {
                            singleResult = compareCheckbox(sourceValue, targetValue);
                            if (singleResult) {
                                break;
                            }
                        } else {
                            //值比较
                            if (StringUtils.isNotEmpty(sourceValue) && StringUtils.isNotEmpty(targetValue) && sourceValue.equals(targetValue)) {
                                singleResult = true;
                                break;
                            }
                        }
                    }
                    if (singleResult) {
                        break;
                    }
                }
                if (!singleResult) {
                    return false;
                }
            }
        } catch (Exception e) {
            flag = false;
        }
        return flag;
    }

    /**
     * json值比较
     */
    private static boolean compareJson(String sourcePropertyFieldValue, String targetPropertyFieldValue) {
        try {
            return JsonCompareUtil.campareJsonString(sourcePropertyFieldValue, targetPropertyFieldValue);
        } catch (Exception e) {
            log.error("json值比较异常", e);
            return false;
        }
    }

    /**
     * 多选字段值比较
     * ["position1","position2"]
     */
    private static boolean compareCheckbox(String sourcePropertyFieldValue, String targetPropertyFieldValue) {
        if (StringUtils.isEmpty(sourcePropertyFieldValue) || StringUtils.isEmpty(targetPropertyFieldValue)) {
            return false;
        }
        List<String> sourceValueList = JSONObject.parseArray(sourcePropertyFieldValue, String.class);
        List<String> targetValueList = JSONObject.parseArray(targetPropertyFieldValue, String.class);
        for (String source : sourceValueList) {
            if (targetValueList.contains(source)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param sourceInstance
     * @param targetInstance
     * @param moduleRelation
     * @param way
     * @param userId
     * @Description: 获取实例关系
     * @Author majiali
     * @CreateTime 2022/6/8 17:15
     */
    public static InstanceRelation getInstanceRelation(Instance sourceInstance, Instance targetInstance, ModuleRelation moduleRelation, String way, Long userId) {
        InstanceRelation relation = new InstanceRelation();
        relation.setRelationCategoryId(moduleRelation.getRelationCategoryId());
        relation.setRelationCategoryCode(moduleRelation.getRelationCategoryCode());
        relation.setRelationId(moduleRelation.getRelationId());
        relation.setRelationName(moduleRelation.getRelationName());
        relation.setModuleRelationId(moduleRelation.getId());
        relation.setModuleRelationName(moduleRelation.getName());
        relation.setSourceModuleId(moduleRelation.getSourceId());
        relation.setSourceModuleCode(moduleRelation.getSourceCode());
        relation.setTargetModuleId(moduleRelation.getTargetId());
        relation.setTargetModuleCode(moduleRelation.getTargetCode());
        relation.setDeleted(false);
        relation.setSourceInstanceId(sourceInstance.getId());
        relation.setSourceInstanceName(getInstanceName(sourceInstance));
        relation.setTargetInstanceId(targetInstance.getId());
        relation.setTargetInstanceName(getInstanceName(targetInstance));
        relation.setCreatorId(userId);
        relation.setGmtCreate(new Date());
        relation.setSourceType(moduleRelation.getSourceType());
        relation.setTargetType(moduleRelation.getTargetType());
        relation.setWay(way);
        return relation;
    }

    /**
     * @param isPositive     是否是正向关系
     * @param sourceInstance
     * @param targetInstance
     * @param moduleRelation
     * @param way
     * @param userId
     * @Description: 获取调和记录
     * @Author majiali
     * @CreateTime 2022/6/8 17:17
     */
    public static InstanceRelationHistoryRecord getInstanceRelationHistory(boolean isPositive, Instance sourceInstance, Instance targetInstance, ModuleRelation moduleRelation, AutoRelationRule relationRule, String way, String status, String type, Long userId, List<InstanceRelation> oldConflictRelations) {
        InstanceRelationHistoryRecord record = new InstanceRelationHistoryRecord();
        //当前实例是源实例
        if (isPositive) {
            record.setName(getInstanceName(sourceInstance));
            record.setInstanceId(sourceInstance.getId());
            record.setModuleId(moduleRelation.getSourceId());
            record.setModuleCode(moduleRelation.getSourceCode());
            record.setModuleName(moduleRelation.getSourceName());
            if (null != targetInstance) {
                record.setRelationInstanceId(targetInstance.getId());
            }
        } else {
            record.setName(getInstanceName(targetInstance));
            record.setInstanceId(targetInstance.getId());
            record.setModuleId(moduleRelation.getTargetId());
            record.setModuleCode(moduleRelation.getTargetCode());
            record.setModuleName(moduleRelation.getTargetName());
            if (null != sourceInstance) {
                record.setRelationInstanceId(sourceInstance.getId());
            }
        }
        InstanceRelationHistoryAlterModel alter = new InstanceRelationHistoryAlterModel();
        //取消关联
        if (RelationHistoryConstant.Type.CANCEL.getValue().equals(type)) {
            record.setAlterAfter("--");
        } else {
            InstanceRelation instanceRelation = AutoModuleInstanceRelationUtils.getInstanceRelation(sourceInstance, targetInstance, moduleRelation, way, userId);
            alter.setAfter(instanceRelation);
            record.setAlterAfter(getInstanceName(sourceInstance) + "->" + getInstanceName(targetInstance));
        }
        record.setRelationCategoryId(moduleRelation.getRelationCategoryId());
        record.setAutoRelationRuleId(relationRule.getId());
        record.setModuleRelationId(moduleRelation.getId());
        record.setIsSource(isPositive);
        if (CollectionUtils.isEmpty(oldConflictRelations)) {
            record.setAlterBefore("--");
        } else {
            alter.setBefore(oldConflictRelations);
            String beforeStr = "";
            for (InstanceRelation old : oldConflictRelations) {
                beforeStr += old.getSourceInstanceName() + "->" + old.getTargetInstanceName() + ",";
            }
            if(StringUtils.isNotEmpty(beforeStr)){
                beforeStr = beforeStr.substring(0, beforeStr.length() - 1);
            }
            record.setAlterBefore(beforeStr);
        }
        record.setProps(JSONObject.toJSONString(alter));
        record.setDeleted(false);
        record.setCreatorId(userId);
        record.setGmtCreate(new Date());
        record.setWay(way);
        record.setStatus(status);
        record.setType(type);
        if (!RelationHistoryConstant.Status.RECONCILED.getValue().equals(status)) {
            record.setExecTime(new Date());
            record.setExecId(userId);
        }
        return record;
    }

    private static String getInstanceName(Instance instance) {
        String name = "";
        List<InstanceValue> instanceValues = instance.getInstanceValues();
        for (InstanceValue value : instanceValues) {
            if ("name".equals(value.getPropertyCode())) {
                name = value.getPropertyValue();
                break;
            }
        }
        return name;
    }

}
